@@ -21,7 +21,6 @@ from collections import defaultdict, namedtuple
 from typing import Callable, Optional, Union
 from urllib.parse import urlparse
 
-import attrdict
 import requests
 import retry
 import six
@@ -76,11 +75,18 @@ class Callback(str, enum.Enum):
     RET_CONTINUE = "::continue"
 
 
+class AttrDict(dict):
+    def __getattr__(self, key):
+        if isinstance(key, str) and key in self:
+            return self[key]
+        raise AttributeError("Attribute key not found", key)
+
+
 def convert(dictionary):
     """
     Convert dict to namedtuple
     """
-    return attrdict.AttrDict(dictionary)
+    return AttrDict(dictionary)
 
     # Old implement
     # return namedtuple('GenericDict', list(dictionary.keys()))(**dictionary)
@@ -112,7 +118,7 @@ def namedlock(name):
     return namedlock.locks[name]
 
 
-def httpdo(url, method="GET", data=None, timeout=None) -> attrdict.AttrDict:
+def httpdo(url, method="GET", data=None, timeout=None) -> AttrDict:
     """
     thread safe http request
 
@@ -160,7 +166,10 @@ def _unsafe_httpdo(url, method='GET', data=None, timeout=None):
 
     if DEBUG:
         ms = (time.time() - start) * 1000
-        print('Return ({:.0f}ms): {}'.format(ms, response.text))
+        response_text = response.text
+        if url.endswith("/screenshot"):
+            response_text = response_text[:100] + "..." # limit length of screenshot response
+        print('Return ({:.0f}ms): {}'.format(ms, response_text))
 
     try:
         retjson = response.json()
@@ -181,7 +190,7 @@ def _unsafe_httpdo(url, method='GET', data=None, timeout=None):
     except JSONDecodeError:
         if response.text == "":
             raise WDAEmptyResponseError(method, url, data)
-        raise WDAError(method, url, response.text)
+        raise WDAError(method, url, response.text[:100] + "...") # should not too long
     except requests.ConnectionError as e:
         raise WDAError("Failed to establish connection to to WDA")
 
@@ -230,23 +239,24 @@ class Rect(list):
 
 
 def _start_wda_xctest(udid: str, wda_bundle_id=None) -> bool:
-    tins_path = shutil.which("tins")
-    if not tins_path:
+    xctool_path = shutil.which("tins2") or shutil.which("tidevice")
+    if not xctool_path:
         return False
-    logger.info("WDA is not running, exec: tins xctest")
+    logger.info("WDA is not running, exec: {} xctest".format(xctool_path))
     args = []
     if udid:
         args.extend(['-u', udid])
     args.append('xctest')
     if wda_bundle_id:
         args.extend(['-B', wda_bundle_id])
-    p = subprocess.Popen([tins_path] + args)
+    p = subprocess.Popen([xctool_path] + args)
     time.sleep(3)
     if p.poll() is not None:
-        logger.warning("tins xctest launch failed")
+        logger.warning("xctest launch failed")
         return False
     return True
 
+
 class BaseClient(object):
     def __init__(self, url=None, _session_id=None):
         """
@@ -257,7 +267,7 @@ class BaseClient(object):
         """
         if not url:
             url = os.environ.get('DEVICE_URL', 'http://localhost:8100')
-        assert re.match(r"^(usbmux|https?)://", url), "Invalid URL: %r" % url
+        assert re.match(r"^(http\+usbmux|https?)://", url), "Invalid URL: %r" % url
 
         # Session variable
         self.__wda_url = url
@@ -272,7 +282,7 @@ class BaseClient(object):
             self._init_callback()
 
         # u = urllib.parse.urlparse(self.__wda_url)
-        # if u.scheme == "usbmux" and not self.is_ready():
+        # if u.scheme == "http+usbmux" and not self.is_ready():
         #     udid = u.netloc.split(":")[0]
         #     if _start_wda_xctest(udid):
         #         self.wait_ready()
@@ -289,7 +299,7 @@ class BaseClient(object):
         """ 等待设备恢复上线 """
 
     def _callback_wait_ready(self, err):
-        logger.warning("Error: %s", err)
+        # logger.warning("Error: %s", err) # too noisy
         if isinstance(err, (ConnectionError, requests.ConnectionError,
                             requests.ReadTimeout, WDABadGateway)):
             if not self.wait_ready(DEVICE_WAIT_TIMEOUT):  # 等待设备恢复在线
@@ -357,7 +367,7 @@ class BaseClient(object):
         def _dprint(message: str):
             if noprint:
                 return
-            print(time.ctime(), message)
+            print("facebook-wda", time.ctime(), message)
 
         _dprint("Wait ready (timeout={:.1f})".format(timeout))
         while time.time() < deadline:
@@ -365,8 +375,7 @@ class BaseClient(object):
                 _dprint("device back online")
                 return True
             else:
-                _dprint("wait_ready left {:.1f} seconds".format(deadline -
-                                                                time.time()))
+                _dprint("{!r} wait_ready left {:.1f} seconds".format(self.__wda_url, deadline - time.time()))
                 time.sleep(1.0)
         _dprint("device still offline")
         return False
@@ -423,7 +432,7 @@ class BaseClient(object):
                urlpath: str,
                data: Optional[dict] = None,
                with_session: bool = False,
-               timeout: Optional[float] = None) -> attrdict.AttrDict:
+               timeout: Optional[float] = None) -> AttrDict:
         """ do http request """
         urlpath = "/" + urlpath.lstrip("/")  # urlpath always startswith /
 
@@ -1057,6 +1066,35 @@ class BaseClient(object):
                 f"Invalid name: {name}, should be one of {valid_names}")
         self._session_http.post("/wda/pressButton", {"name": name})
 
+    def press_duration(self, name: str, duration: float):
+        """
+        Args:
+            name: one of <home|volumeUp|volumeDown|power|snapshot>
+            duration: seconds
+
+        Notes:
+            snapshot equals power+home
+
+        Raises:
+            ValueError
+
+        Refs:
+            https://github.com/appium/WebDriverAgent/pull/494/files
+        """
+        hid_usages = {
+            "home": 0x40,
+            "volumeup": 0xE9,
+            "volumedown": 0xEA,
+            "power": 0x30,
+            "snapshot": 0x65,
+            "power+home": 0x65
+        }
+        name = name.lower()
+        if name not in hid_usages:
+            raise ValueError("Invalid name:", name)
+        hid_usage = hid_usages[name]
+        return self._session_http.post("/wda/performIoHidEvent", {"page": 0x0C, "usage": hid_usage, "duration": duration})
+
     def keyboard_dismiss(self):
         """
         Not working for now
@@ -1765,13 +1803,10 @@ class USBClient(Client):
                 raise RuntimeError("more then one device connected")
             udid = infos[0]['SerialNumber']
 
-        super().__init__(url="usbmux://{}:{}".format(udid, port))
+        super().__init__(url=requests_usbmux.DEFAULT_SCHEME + "{}:{}".format(udid, port))
         if self.is_ready():
             return
 
-        if shutil.which("tins"):
-            ok = _start_wda_xctest(udid, wda_bundle_id)
-            if not ok:
-                raise RuntimeError("init wda xctest error")
-            if not self.wait_ready(timeout=20):
-                raise RuntimeError("wda xctest launched but check failed")
+        _start_wda_xctest(udid, wda_bundle_id)
+        if not self.wait_ready(timeout=20):
+            raise RuntimeError("wda xctest launched but check failed")
